﻿using System;
using System.Net;
using Inedo.Agents.Hosting;
using Inedo.WindowsServices;

namespace Inedo.Agents.Service
{
    internal sealed class AgentService : InedoService
    {
        private const string DefaultServiceName = "INEDOAGENTSVC";
        private InedoAgentServer server;

        public AgentService()
        {
            HostedProcessManager.Instance = new HostedProcessManager();
        }

        public static AgentService Instance { get; } = new AgentService();

        public override string Name => DefaultServiceName;

        protected override void OnStart()
        {
            ValidateConfiguration();
            this.server = new InedoAgentServer(IPAddress.Any, AgentConfig.Port);
            var rubbish = this.server.WaitForConnectionsAsync();
        }
        protected override void OnStop()
        {
            this.server?.Dispose();
            this.server = null;
        }

        private static void ValidateConfiguration()
        {
            switch (AgentConfig.Encryption)
            {
                case AgentEncryptionMode.Ssl:
                    var certificateName = AgentConfig.CertificateName;
                    if (string.IsNullOrWhiteSpace(certificateName))
                        throw new InvalidOperationException("SSL encryption is specified, but CertificateName is not set.");
                    break;

                case AgentEncryptionMode.Aes:
                    var key = AgentConfig.EncryptionKey;
                    if (key == null || key.Length == 0)
                        throw new InvalidOperationException("AES encryption is specified, but EncryptionKey is not set.");
                    if (key.Length != 16)
                        throw new InvalidOperationException("AES encryption is specified, but EncryptionKey is invalid.");
                    break;
            }
        }
    }
}
